package it.eng.spago.cache;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.coordinator.AbstractCoordinator;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.tracing.TracerSingleton;

public class CacheResponseHandler {

	public static final String REACH_SERVICE = "REACH_SERVICE";
	public static final String CACHED_RESPONSE = "CACHED_RESPONSE";
	public static final String CACHED_RESPONSE_SB = "CACHED_RESPONSE_SB";
	
	public static void service(RequestContainer requestContainer, ResponseContainer responseContainer, String publisher){
		
		EMFErrorHandler errorHandler = responseContainer.getErrorHandler();
		if ((errorHandler == null) || (errorHandler.isOK())){
			String businessName = (String)requestContainer.getAttribute(AbstractCoordinator.CURRENT_SERVICE_NAME);	
			String businessType = (String)requestContainer.getAttribute(AbstractCoordinator.CURRENT_SERVICE_TYPE);
		
			ConfigSingleton configure = ConfigSingleton.getInstance();
			SourceBean serviceSourceBean = null;
			if (businessType.equalsIgnoreCase("PAGE")){
		        serviceSourceBean =
		            (SourceBean) configure.getFilteredSourceBeanAttribute(
		                "PAGES.PAGE",
		                "NAME",
		                businessName);
			}else{
				serviceSourceBean = (SourceBean)configure.getFilteredSourceBeanAttribute(
						"ACTIONS.ACTION", 
						"NAME", 
						businessName);
			}
			
			if ( serviceSourceBean.getAttribute("REACH-SERVICES") != null ){
				SessionContainer permanentContainer = requestContainer.getSessionContainer().getPermanentContainer();
			
				try{
					SourceBean cachedResponseSourceBean = getCachedResponse(permanentContainer);
				
					//Cache the service Response
					if (cachedResponseSourceBean.getAttribute(publisher) != null){
						cachedResponseSourceBean.updAttribute(publisher, responseContainer.getServiceResponse());
					}else{
						cachedResponseSourceBean.setAttribute(publisher, responseContainer.getServiceResponse());
					}
					
				}catch (Exception e) {
					TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
		            "CacheResponseHandler::service: ", e);
				}
			}
			
		}
		
	}
	
	public static SourceBean getCachedResponse(SessionContainer permanentContainer) throws SourceBeanException {
		
			if (permanentContainer.getAttribute(CACHED_RESPONSE) == null){
				SourceBean sb = new SourceBean(CACHED_RESPONSE_SB);
				permanentContainer.setAttribute(CACHED_RESPONSE, sb);
			}
		
			return (SourceBean)permanentContainer.getAttribute(CACHED_RESPONSE);
	
		
	}
}
