/**

Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

This file is part of Spago.

Spago is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
any later version.

Spago is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Spago; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.cache;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.coordinator.AbstractCoordinator;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.presentation.AbstractPublisherDispatcher;

import java.util.List;

public class DynamicCacheResponsePublisher extends AbstractPublisherDispatcher {
	/**
	 *
	 */
	public DynamicCacheResponsePublisher() {
		super();

	}

	public String getPublisherName(RequestContainer requestContainer,
			ResponseContainer responseContainer) {
	
		EMFErrorHandler errorHandler = responseContainer.getErrorHandler();
		String businessName = (String)requestContainer.getAttribute(AbstractCoordinator.CURRENT_SERVICE_NAME);
//		String businessType = (String)requestContainer.getAttribute(AbstractCoordinator.CURRENT_SERVICE_TYPE);
		String targetPublisher  = ((SourceBean)(getConfig().getAttribute("TARGET_PUBLISHER"))).getCharacters();
		String lastPublisherName = getLastPublisherName(requestContainer);
		if ((errorHandler != null) && (!errorHandler.isOK())){
			// Si sono verificati errori nell'esecuzione del servizio
			
			SourceBean cachedResponseForPublisher = getCachedResponseForPublisher(requestContainer, lastPublisherName);
			if ((isReachable(businessName)) && ( cachedResponseForPublisher != null)){
				responseContainer.setServiceResponse(cachedResponseForPublisher);
			}
			return lastPublisherName;
		}else{
			return targetPublisher;
		}
	}
	
	
	public boolean isReachable(String businessName){
		
		List isReachableList = (List)ApplicationContainer.getInstance().getAttribute("REACHABLE_LIST");
		
		return isReachableList.contains(businessName);
	}
	
	public String getLastPublisherName(RequestContainer requestContainer){
		
		return (String)requestContainer.getSessionContainer().getPermanentContainer().getAttribute("LAST_PUBLISHER_NAME");
		
	}
	
	public SourceBean getCachedResponseForPublisher(RequestContainer requestContainer, String publisherName){
	
		SessionContainer permanentContainer = requestContainer.getSessionContainer().getPermanentContainer(); 
		
		SourceBean cachedResponses =  (SourceBean)permanentContainer.getAttribute(CacheResponseHandler.CACHED_RESPONSE);
		
		if (cachedResponses != null){
			
			SourceBean sb = (SourceBean)cachedResponses.getAttribute(publisherName);
			return (SourceBean)sb.getAttribute(Constants.SERVICE_RESPONSE);
		}else{
			
			return null;
		
		}
	}
	
}

